function[beta,sebeta,eta,pvA3,pvH,lambdahcv,lambdagcv]=TSIV(Y,X,Z,p1,type,J1,K1,lambdah,lambdag)
%% Two-step IV for OLIVA
% Author: Juan Carlos Escanciano
% Date: Jan 2nd 2020
% First-step solves E[h(Z)/X2]=X2. 
% Data: Y dependent, X (endog) and Z (instr) do not contain intercepts
% p1 =#exogenous controls 
% Sieves: type=1 Power series or type=2 B-Splines with K-terms for X and J-terms for Z.
% J1 individual number of terms for instruments (without intercept) J=q2*J1+1;
% K1 individual number of terms for endognous (without intercept) K=p2*K1+1;
% lambdah is user provided if not zero, or lambdah=gcv if lambdah=0
% Output: OLIVA estimate(beta), s.e, eta (instrument on endog) with pval pvA3, pval for Hausman, cv lambdas
%% Z=instruments, X=endogenous, p1<p exogenous controls
[n,p]=size(X);
[n,q]=size(Z);
p2=p-p1; %p2 number of endogenous variables
q2=q-p1; %q2 number of instrumental variables
q1=p1;
Z1=Z(:,1:q1);
Z2=Z(:,(q1+1):q);
X1=Z1;
X2=X(:,(p1+1):p);
% in the IES application p1=q1=0,p2=1,q2=4
%% Construct matrices with sieves (additive as of now)
J=q2*J1;
K=p2*K1;
Q=zeros(n,J);
P=zeros(n,K);
% For cubic splines J1,K1 larger than 4
%type=1; %=1 for series =2 for splines
[P Q]=sieves(X,Z,p1,J1,K1,type);
% qhat nxJ matrix
qhat=zeros(n,J);
ProP=zeros(n,n);
ProP=P*(pinv(P'*P))*P';
qhat=ProP*Q; 
% First-step estimator
Q2n=Q'*Q;
Dn=(qhat'*X2)./n;
Bxn=(Q'*X)./n;
Byn=(Q'*Y)./n;
Xe=[ones(n,1) X];
% if lambdah=0 run loocv
if (lambdah==0)
    lambdagrid=linspace(0.001,0.1); 
    ngrid=length(lambdagrid);
    for j=1:ngrid
        lambdaux=lambdagrid(j);
        Alamb=(qhat'*qhat+lambdaux*Q2n)./n;
        Alambinv=pinv(Alamb);        
        hhat=Q*Alambinv*Dn;
        Hn=[ones(n,1) hhat];
        Llambda=Xe*inv(Hn'*Xe)*Hn';
        vlambda=trace(Llambda);
        Ylambda=Llambda*Y;
        score(j)=mean((Y-Ylambda/(1-(vlambda/n))).^2);
    end
    [a,jopt]=min(score);
    lambdahcv=lambdagrid(jopt);
else 
    lambdahcv=lambdah;
end

Alamb=(qhat'*qhat+lambdahcv*Q2n)./n;
Alambinv=pinv(Alamb);
hhat=Q*Alambinv*Dn;
eta=inv(X'*X)*X'*hhat;  % eta for checking Assumption 3
Hn=[ones(n,1) hhat];
Xe=[ones(n,1) X];
% TSIV estimator
beta=inv(Hn'*Xe)*Hn'*Y;
%%%%%%%%%%%%%%%% Estimating ghat and ahat
% ahat is used to check Assumption 3 in the paper
[Pg Qg]=sieves(X,Z,p1,J1,K1,type);
P2n=Pg'*Pg;
ProQ=Qg*(pinv(Qg'*Qg))*Qg';
phat=ProQ*Pg;
% if lambdag=0 run loocv
if (lambdag==0)
    lambdagrid=linspace(0.001,0.1); 
    ngrid=length(lambdagrid);
    for j=1:ngrid
        lambdaux=lambdagrid(j);
        Blamb=(phat'*phat+lambdaux*P2n);
        Blambinv=pinv(Blamb);
        Llambda=Pg*Blambinv*Pg'*ProQ;
        vlambda=trace(Llambda);
        Ylambda=Llambda*Y;
        score(j)=mean((Y-Ylambda/(1-(vlambda/n))).^2);
    end
    [a,jopt]=min(score);
    lambdagcv=lambdagrid(jopt);
else 
    lambdagcv=lambdag;
end

Blamb=(phat'*phat+lambdagcv*P2n)./n;
Blambinv=pinv(Blamb);
rhat=ProQ*Y;
En=(phat'*rhat)./n;
Fn=(phat'*hhat)./n;
ghat=Pg*Blambinv*En;
ahat=Pg*Blambinv*Fn;
[Xsort ind]=sort(X);
ghatsort=ghat(ind);
plot(Xsort,ghatsort)
pause
% TSIV Variance Estimation
Yfit=Xe*beta;
res=Y-Yfit;
mis=ghat-Yfit;
Mhat=diag(res)*Hn-diag(mis)*(Hn-Xe);
Vhat=inv(Hn'*Xe)*Mhat'*Mhat*inv(Hn'*Xe);
sebeta=sqrt(diag(Vhat));
% eta Variance Estimation
hhatfit=X*eta;
reshat=hhat-hhatfit;
Mehat=diag(reshat)*X-diag(ahat)*(hhat-X);
Vehat=inv(X'*X)*Mehat'*Mehat*inv(X'*X);
seeta=sqrt(diag(Vehat));
A3test=(eta-1)./seeta;
pvA3=2*normcdf(-abs(A3test));
%%%% Hausman test
% First stage residuals
beta1=inv(Hn'*Hn)*Hn'*X;
Xfit=Hn*beta1;
uhat=X-Xfit;
% Regression implmentation of Hausman test
He=[ones(n,1) X uhat];
gamOLS=inv(He'*He)*He'*Y;
Hres=Y-He*gamOLS;
VHhatOLS=inv(He'*He)*He'*diag(Hres.^2)*He*inv(He'*He);
seHOLS=sqrt(diag(VHhatOLS));
Httest=gamOLS(3,1)./seHOLS(3);
pvH=2*normcdf(-abs(Httest));

